﻿using System;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Timers;
using System.Linq;

class MyServer
{
    private static string[] records;
    private static int currentIndex = 0;
    private const string END_MESSAGE = "ВСЕ_ЗАПИСИ_ВЫДАНЫ";
    private static System.Timers.Timer saveTimer;
    private const string SAVE_FILE_NAME = "last.txt";
    private const string IMAP_FILE_NAME = "accounts.txt";

    
    static async Task Main()
    {
        // Загрузка записей из файла
        LoadRecords(IMAP_FILE_NAME);

        // Настройка таймера для сохранения
        SetupSaveTimer();

        // Настройка сервера
        int port = 8888;
        TcpListener server = new TcpListener(IPAddress.Any, port);
        server.Start();
        Console.WriteLine($"Сервер запущен на порту {port}");

        while (true)
        {
            TcpClient client = await server.AcceptTcpClientAsync(); // ждём запрос
            _ = HandleClientAsync(client); // асинхронная обработка запроса
        }
    }

    static void LoadRecords(string filename)
    {
        records = File.ReadAllLines(filename);       
        Console.WriteLine($"Загружено {records.Length} записей");
    }

   
    static void SetupSaveTimer()
    {
        saveTimer = new System.Timers.Timer(5 * 60 * 1000); // 5 минут
        saveTimer.Elapsed += SaveRemainingRecords;
        saveTimer.AutoReset = true;
        saveTimer.Enabled = true;
    }

    static void SaveRemainingRecords(object sender, ElapsedEventArgs e)
    {
        int remainingCount = records.Length - currentIndex;
        if (remainingCount > 0)
        {
            string[] remainingRecords = new string[remainingCount];
            Array.Copy(records, currentIndex, remainingRecords, 0, remainingCount);
            File.WriteAllLines(SAVE_FILE_NAME, remainingRecords);
            Console.WriteLine($"Сохранено {remainingCount} записей в файл {SAVE_FILE_NAME}");
        }
        else
        {
            File.WriteAllText(SAVE_FILE_NAME, END_MESSAGE);
            Console.WriteLine($"Все записи выданы. Сохранено сообщение о конце в файл {SAVE_FILE_NAME}");
        }
    }

    static async Task HandleClientAsync(TcpClient client) // обработка запроса клиента в асинхронном режиме
    {
        using (NetworkStream stream = client.GetStream())
        {
            string record = await GetNextRecordAsync();
            byte[] data = Encoding.UTF8.GetBytes(record);
            await stream.WriteAsync(data, 0, data.Length);
        }
        client.Close();
    }

    static async Task<string> GetNextRecordAsync()
    {
        int index = Interlocked.Increment(ref currentIndex) - 1;
        if (index < records.Length)
        {
            return records[index];
        }
        else
        {
            // Если записи закончились, формируем файл заново           
            LoadRecords(IMAP_FILE_NAME);
            currentIndex = 0;
            return await GetNextRecordAsync(); // вызов для получения первой записи из нового набора
        }
    }
}
